package com.book.backend.service.impl;

import cn.hutool.core.date.StopWatch;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.book.backend.common.R;
import com.book.backend.common.exception.VueBookException;
import com.book.backend.manager.GuavaRateLimiterManager;
import com.book.backend.mapper.AiIntelligentMapper;
import com.book.backend.pojo.AiIntelligent;
import com.book.backend.pojo.Books;
import com.book.backend.pojo.UserInterfaceInfo;
import com.book.backend.service.AiIntelligentService;
import com.book.backend.service.BooksService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * @author xiaobaitiao
 * @description 针对表【t_ai_intelligent】的数据库操作Service实现
 * @createDate 2023-08-27 18:44:26
 */
@Service
public class AiIntelligentServiceImpl extends ServiceImpl<AiIntelligentMapper, AiIntelligent>
        implements AiIntelligentService {

    @Resource
    private BooksService booksService;
    @Resource
    private GuavaRateLimiterManager guavaRateLimiterManager;
    @Resource
    private UserInterfaceInfoServiceImpl userInterfaceInfoService;
    @Resource
    @Lazy
    private AiIntelligentService aiIntelligentService;

    // 创建RestTemplate对象，用于发送HTTP请求
    private final RestTemplate restTemplate;

    // 在构造方法中初始化RestTemplate并设置超时
    public AiIntelligentServiceImpl() {
        org.springframework.http.client.SimpleClientHttpRequestFactory factory = new org.springframework.http.client.SimpleClientHttpRequestFactory();
        factory.setConnectTimeout(600000);
        factory.setReadTimeout(600000);
        this.restTemplate = new RestTemplate(factory);
    }

    // 设置NewAPI的OpenAI格式接口信息
    private static final String NEW_API_URL = "https://openai.933999.xyz/v1/chat/completions";
    private static final String API_KEY = "sk-1PBIyxIdJ42yyC11XRNqbEXYDt2eZRNVNbd8XxmKjnPXGh5S";
    private static final String MODEL = "gpt-4o-mini";


    @Override
    public R<String> getGenResult(AiIntelligent aiIntelligent) {
        System.out.println("==========开始AI推荐请求处理==========");
        // 判断用户输入文本是否过长，超过128字，直接返回，防止资源耗尽
        String message = aiIntelligent.getInputMessage();
        if(StringUtils.isBlank(message)){
            return R.error("文本不能为空");
        }
        if (message.length() > 128) {
            return R.error("文本字数过长");
        }
        Long user_id = aiIntelligent.getUserId();
        // 查看用户接口次数是否足够，如果不够直接返回接口次数不够
        LambdaQueryWrapper<UserInterfaceInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserInterfaceInfo::getUserId, user_id);
        queryWrapper.eq(UserInterfaceInfo::getInterfaceId, 1);
        UserInterfaceInfo userInterfaceInfo = userInterfaceInfoService.getOne(queryWrapper);
        if (userInterfaceInfo == null) {
            return R.error("该接口已废弃");
        }
        Integer leftNum = userInterfaceInfo.getLeftNum();
        Integer totalNum = userInterfaceInfo.getTotalNum();
//        if (leftNum <= 0) {
//            return R.error("AI接口次数不足，请明天再来");
//        }
        // 限流
        boolean rateLimit = guavaRateLimiterManager.doRateLimit(user_id);
//        if (!rateLimit) {
//            return R.error("请求次数过多，请稍后重试");
//        }

        // 准备发送给AI的信息
        List<Books> list = booksService.list();
        StringBuilder stringBuilder = new StringBuilder();
        HashSet<String> hashSet = new HashSet<>();
        String presetInformation = "请根据数据库内容和游客信息作出推荐,书籍优先选择数据库里面有的，如果游客喜欢的书籍，数据库没有，你可能根据自身的知识去推荐，可以是一本也可以是多本，但不可以超过三本书，根据游客喜欢的信息作出推荐，输出的文字格式都为普通格式，不要出现markdown格式。如果用户问的问题与图书推荐无关，请拒绝回答！";

        stringBuilder.append(presetInformation).append("\n").append("数据库内容: ");
        for (Books books : list) {
            if (!hashSet.contains(books.getBookName())) {
                hashSet.add(books.getBookName());
                stringBuilder.append(books.getBookName()).append(",");
            }
        }
        stringBuilder.append("\n");

        stringBuilder.append("游客信息: ").append(message).append("\n");

        // 调用之前先获取该用户最近的五条历史记录
        R<List<AiIntelligent>> history = aiIntelligentService.getAiInformationByUserId(user_id);
        List<AiIntelligent> historyData = history.getData();

        String response;
        ExecutorService executor = Executors.newSingleThreadExecutor();

        // 转换为NewAPI OpenAI格式的消息
        List<Map<String, String>> messages = new ArrayList<>();

        // 添加系统角色消息
        Map<String, String> systemMessage = new HashMap<>();
        systemMessage.put("role", "system");
        systemMessage.put("content", presetInformation);
        messages.add(systemMessage);

        // 添加历史消息
        if (historyData != null) {
            for (AiIntelligent item : historyData) {
                Map<String, String> userMessage = new HashMap<>();
                userMessage.put("role", "user");
                userMessage.put("content", item.getInputMessage());
                messages.add(userMessage);

                Map<String, String> assistantMessage = new HashMap<>();
                assistantMessage.put("role", "assistant");
                assistantMessage.put("content", item.getAiResult());
                messages.add(assistantMessage);
            }
        }

        // 添加当前用户消息
        Map<String, String> currentUserMessage = new HashMap<>();
        currentUserMessage.put("role", "user");
        currentUserMessage.put("content", stringBuilder.toString());
        messages.add(currentUserMessage);

        // 构造请求
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("model", MODEL);
        requestBody.put("messages", messages);
        requestBody.put("max_tokens", 2048);
        requestBody.put("temperature", 0.2);

        // 打印请求信息
        System.out.println("===== 请求信息 =====");
        System.out.println("API地址: " + NEW_API_URL);
        System.out.println("模型: " + MODEL);
        System.out.println("消息数量: " + messages.size());
        try {
            // 使用Jackson打印请求体JSON内容
            com.fasterxml.jackson.databind.ObjectMapper mapper = new com.fasterxml.jackson.databind.ObjectMapper();
            System.out.println("请求体: " + mapper.writeValueAsString(requestBody));
        } catch (Exception e) {
            System.out.println("无法打印请求体: " + e.getMessage());
        }

        // 设置HTTP头部
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("Authorization", "Bearer " + API_KEY);

        HttpEntity<Map<String, Object>> requestEntity = new HttpEntity<>(requestBody, headers);

        int timeout = 60;
        Future<String> future = executor.submit(() -> {
            try {
                // 同步调用
                StopWatch stopWatch = new StopWatch();
                stopWatch.start();

                // 发送请求到NewAPI
                System.out.println("开始发送请求到API...");
                Map<String, Object> responseBody = null;
                try {
                    responseBody = restTemplate.postForObject(NEW_API_URL, requestEntity, Map.class);
                    System.out.println("API请求成功，开始解析响应");
                    // 打印完整响应
                    com.fasterxml.jackson.databind.ObjectMapper mapper = new com.fasterxml.jackson.databind.ObjectMapper();
                    System.out.println("完整响应: " + mapper.writeValueAsString(responseBody));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println("API请求失败: " + e.getMessage());
                    System.out.println("异常类型: " + e.getClass().getName());
                    if (e.getCause() != null) {
                        System.out.println("异常原因: " + e.getCause().getMessage());
                    }
                    return "抱歉，服务暂时不可用，请稍后再试。错误: " + e.getMessage();
                }

                stopWatch.stop();
                long total = stopWatch.getTotal(TimeUnit.SECONDS);
                System.out.println("本次接口调用耗时:" + total + "秒");

                // 解析响应
                @SuppressWarnings("unchecked")
                List<Map<String, Object>> choices = (List<Map<String, Object>>) responseBody.get("choices");
                if (choices != null && !choices.isEmpty()) {
                    Map<String, Object> choice = choices.get(0);
                    @SuppressWarnings("unchecked")
                    Map<String, String> messageObj = (Map<String, String>) choice.get("message");
                    String content = messageObj.get("content");

                    // 打印token使用情况
                    @SuppressWarnings("unchecked")
                    Map<String, Object> usage = (Map<String, Object>) responseBody.get("usage");
                    if (usage != null) {
                        System.out.println("\n提问tokens：" + usage.get("prompt_tokens")
                                + "，回答tokens：" + usage.get("completion_tokens")
                                + "，总消耗tokens：" + usage.get("total_tokens"));
                    }

                    return content;
                }
                return "未能获取到有效回答";
            } catch (Exception exception) {
                exception.printStackTrace();
                throw new RuntimeException("遇到异常: " + exception.getMessage());
            }
        });

        try {
            System.out.println("等待Future任务完成，超时设置: " + timeout + "秒");
            response = future.get(timeout, TimeUnit.SECONDS);
            System.out.println("Future任务成功完成，返回结果: " + (response != null ? response.substring(0, Math.min(50, response.length())) + "..." : "null"));
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("Future任务执行异常: " + e.getMessage() + ", 类型: " + e.getClass().getName());
            future.cancel(true); // 取消任务
            return R.error("服务器内部错误，请求超时，请稍后重试 (" + e.getMessage() + ")");
        } finally {
            // 关闭ExecutorService
            System.out.println("关闭执行器");
            executor.shutdownNow(); // 使用shutdownNow强制关闭执行器
        }

        // 保存结果
        AiIntelligent saveResult = new AiIntelligent();
        saveResult.setInputMessage(aiIntelligent.getInputMessage());
        saveResult.setAiResult(response);
        saveResult.setUserId(user_id);
        boolean save = this.save(saveResult);
        if (!save) {
            throw new VueBookException("获取AI推荐信息失败");
        }

        // 更新调用接口的次数 剩余接口调用次数-1.总共调用次数+1
        userInterfaceInfo.setLeftNum(leftNum - 1);
        userInterfaceInfo.setTotalNum(totalNum + 1);
        boolean update = userInterfaceInfoService.updateById(userInterfaceInfo);
        if (!update) {
            return R.error("调用接口信息失败");
        }
        return R.success(response, "获取AI推荐信息成功");
    }

    @Override
    public R<List<AiIntelligent>> getAiInformationByUserId(Long userId) {
        QueryWrapper<AiIntelligent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(userId != null && userId > 0, "user_id", userId);
        queryWrapper.orderByDesc("create_time");
        queryWrapper.last("LIMIT 5");
        List<AiIntelligent> list = this.list(queryWrapper);
        if (list.size() == 0) {
            return R.success(null, "用户暂时没有和AI的聊天记录");
        }
        Collections.reverse(list);
        return R.success(list, "获取和AI最近的5条聊天记录成功");
    }

    public static int getSleepTimeStrategy(String message){
        int length = message.length();
        if(length<20){
            return 10;
        }else if(length<=30){
            return 12;
        }else if(length<=50){
            return 15;
        }else{
            return 20;
        }
    }
}