package com.book.backend.service.impl;

import cn.hutool.core.io.FileUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.book.backend.common.R;
import com.book.backend.common.exception.ErrorCode;
import com.book.backend.common.exception.ThrowUtils;
import com.book.backend.manager.AiPost;
import com.book.backend.pojo.Admins;
import com.book.backend.pojo.Chart;
import com.book.backend.pojo.UserInterfaceInfo;
import com.book.backend.pojo.dto.chart.GenChartByAiRequest;
import com.book.backend.pojo.vo.BiResponse;
import com.book.backend.service.AdminsService;
import com.book.backend.service.ChartService;
import com.book.backend.mapper.ChartMapper;
import com.book.backend.utils.ExcelUtils;
import com.google.gson.Gson;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class ChartServiceImpl extends ServiceImpl<ChartMapper, Chart> implements ChartService {

    @Resource
    private AdminsService adminsService;
    @Resource
    private AiPost aiPost;
    @Resource
    private UserInterfaceInfoServiceImpl userInterfaceInfoService;

    @Override
    public R<BiResponse> genChartByAi(MultipartFile multipartFile, GenChartByAiRequest genChartByAiRequest) {
        String name = genChartByAiRequest.getName();
        String goal = genChartByAiRequest.getGoal();
        String chartType = genChartByAiRequest.getChartType();

        // 参数校验
        ThrowUtils.throwIf(StringUtils.isBlank(goal), ErrorCode.PARAMS_ERROR, "目标为空");
        ThrowUtils.throwIf(StringUtils.isNotBlank(name) && name.length() > 100, ErrorCode.PARAMS_ERROR, "名称过长");

        // 文件校验
        long size = multipartFile.getSize();
        String originalFilename = multipartFile.getOriginalFilename();
        final long ONE_MB = 1024 * 1024L;
        ThrowUtils.throwIf(size > ONE_MB, ErrorCode.PARAMS_ERROR, "文件超过 1M");

        String suffix = FileUtil.getSuffix(originalFilename);
        final List<String> validFileSuffixList = Arrays.asList("xlsx", "xls");
        ThrowUtils.throwIf(!validFileSuffixList.contains(suffix), ErrorCode.PARAMS_ERROR, "文件后缀非法");

        // 查询接口配额
        Admins admin = adminsService.getById(genChartByAiRequest.getAdminId());
        LambdaQueryWrapper<UserInterfaceInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserInterfaceInfo::getUserId, admin.getAdminId());
        UserInterfaceInfo interfaceInfo = userInterfaceInfoService.getOne(queryWrapper);

        if (interfaceInfo == null) {
            return R.error("该接口已废弃");
        }

        Integer leftNum = interfaceInfo.getLeftNum();
        Integer totalNum = interfaceInfo.getTotalNum();

        // 构建 AI 输入
        final String prompt = "你是一个数据分析师和前端开发专家，按照以下固定格式分析数据：\n" +
                "分析需求：\n{数据分析的需求或者目标}\n" +
                "原始数据：\n{csv格式的原始数据，用,作为分隔符}\n" +
                "请按以下格式生成内容（不要输出任何多余的开头、结尾、注释）\n" +
                "【【【【【\n" +
                "{前端 Echarts V5 的 option 配置对象js代码，合理地将数据进行可视化}\n" +
                "【【【【【\n" +
                "{明确的数据分析结论，越详细越好}\n";

        StringBuilder userInput = new StringBuilder();
        userInput.append(prompt);
        userInput.append("你是一个数据分析师，接下来我会给你分析目标和原始数据，请告诉我分析结论。").append("\n");

        String userGoal = goal;
        if (StringUtils.isNotBlank(chartType)) {
            userGoal += "，请使用" + chartType;
        }

        userInput.append("分析目标：").append(userGoal).append("\n");
        userInput.append("分析需求：").append(userGoal).append("\n");
        userInput.append("原始数据: ").append("\n");

        String csvData = ExcelUtils.excelToCsv(multipartFile);
        userInput.append(csvData).append("\n");

        String result;
        try {
            result = aiPost.sendMessageAndGetResponse(userInput.toString(), 1024);
            System.out.println("AI返回内容：\n" + result);
        } catch (Exception e) {
            throw new RuntimeException("AI分析失败", e);
        }

        // 解析AI返回内容
        String[] splits = result.split("【【【【【");
        if (splits.length < 3) {
            return R.error("AI生成格式错误，请确认返回内容完整");
        }

        // 提取 option JSON
        String optionBlock = splits[1].trim();
        Pattern jsonPattern = Pattern.compile("\\{.*\\}", Pattern.DOTALL);
        Matcher jsonMatcher = jsonPattern.matcher(optionBlock);
        String genChart;

        if (jsonMatcher.find()) {
            genChart = jsonMatcher.group(0).trim();
        } else {
            return R.error("AI返回的图表配置无法识别");
        }

        // 提取分析结论
        String genResult = splits[2].trim();

        // 解析 Echarts 配置
        Gson gson = new Gson();
        Object objectChart;
        try {
            objectChart = gson.fromJson(genChart, Object.class);
        } catch (Exception e) {
            return R.error("图表配置JSON格式错误：" + e.getMessage());
        }

        // 保存图表数据
        Chart chart = new Chart();
        chart.setName(name);
        chart.setGoal(goal);
        chart.setChartData(csvData);
        chart.setChartType(chartType);
        chart.setGenChart(genChart);
        chart.setGenResult(genResult);
        chart.setAdminId(genChartByAiRequest.getAdminId());

        boolean saveResult = this.save(chart);
        ThrowUtils.throwIf(!saveResult, ErrorCode.SYSTEM_ERROR, "图表保存失败");

        // 更新配额
        interfaceInfo.setLeftNum(leftNum - 1);
        interfaceInfo.setTotalNum(totalNum + 1);
        boolean update = userInterfaceInfoService.updateById(interfaceInfo);
        if (!update) {
            return R.error("调用接口失败");
        }

        // 构造响应
        BiResponse biResponse = new BiResponse();
        biResponse.setGenChart(genChart);
        biResponse.setGenResult(genResult);
        biResponse.setChartId(chart.getId());

        R<BiResponse> resultData = new R<>();
        resultData.add("genChart", objectChart);
        resultData.setData(biResponse);
        resultData.setMsg("图表生成成功");
        resultData.setStatus(200);

        return resultData;
    }
}
